"""Initial table creation

Peek Plugin Database Migration Script

Revision ID: d94277dd8aca
Revises: 
Create Date: 2017-06-25 14:53:23.543928

"""

# revision identifiers, used by Alembic.
revision = "d94277dd8aca"
down_revision = None
branch_labels = None
depends_on = None

import sqlalchemy as sa
from alembic import op


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "DeviceInfo",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("description", sa.String(length=100), nullable=False),
        sa.Column("deviceId", sa.String(length=50), nullable=False),
        sa.Column("deviceType", sa.String(length=20), nullable=False),
        sa.Column("deviceToken", sa.String(length=50), nullable=False),
        sa.Column("appVersion", sa.String(length=15), nullable=False),
        sa.Column("updateVersion", sa.String(length=15), nullable=True),
        sa.Column("lastOnline", sa.DateTime(), nullable=True),
        sa.Column("lastUpdateCheck", sa.DateTime(), nullable=True),
        sa.Column("createdDate", sa.DateTime(), nullable=False),
        sa.Column("isOnline", sa.Boolean(), server_default="0", nullable=False),
        sa.Column(
            "isEnrolled", sa.Boolean(), server_default="0", nullable=False
        ),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("description"),
        sa.UniqueConstraint("deviceId"),
        sa.UniqueConstraint("deviceToken"),
        schema="core_device",
    )
    op.create_table(
        "DeviceUpdate",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("deviceType", sa.String(length=20), nullable=False),
        sa.Column("description", sa.String(), nullable=False),
        sa.Column("buildDate", sa.DateTime(), nullable=False),
        sa.Column("appVersion", sa.String(length=15), nullable=False),
        sa.Column("updateVersion", sa.String(length=15), nullable=False),
        sa.Column("filePath", sa.String(length=150), nullable=False),
        sa.Column("urlPath", sa.String(length=150), nullable=False),
        sa.Column(
            "isEnabled", sa.Boolean(), server_default="0", nullable=False
        ),
        sa.PrimaryKeyConstraint("id"),
        schema="core_device",
    )
    op.create_index(
        "idx_DeviceUpdate_Version",
        "DeviceUpdate",
        ["deviceType", "appVersion", "updateVersion"],
        unique=True,
        schema="core_device",
    )
    op.create_table(
        "Setting",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("name", sa.String(), nullable=True),
        sa.PrimaryKeyConstraint("id"),
        schema="core_device",
    )
    op.create_table(
        "SettingProperty",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("settingId", sa.Integer(), nullable=False),
        sa.Column("key", sa.String(length=50), nullable=False),
        sa.Column("type", sa.String(length=16), nullable=True),
        sa.Column("int_value", sa.Integer(), nullable=True),
        sa.Column("char_value", sa.String(), nullable=True),
        sa.Column("boolean_value", sa.Boolean(), nullable=True),
        sa.ForeignKeyConstraint(
            ["settingId"],
            ["core_device.Setting.id"],
        ),
        sa.PrimaryKeyConstraint("id"),
        schema="core_device",
    )
    op.create_index(
        "idx_SettingProperty_settingId",
        "SettingProperty",
        ["settingId"],
        unique=False,
        schema="core_device",
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index(
        "idx_SettingProperty_settingId",
        table_name="SettingProperty",
        schema="core_device",
    )
    op.drop_table("SettingProperty", schema="core_device")
    op.drop_table("Setting", schema="core_device")
    op.drop_index(
        "idx_DeviceUpdate_Version",
        table_name="DeviceUpdate",
        schema="core_device",
    )
    op.drop_table("DeviceUpdate", schema="core_device")
    op.drop_table("DeviceInfo", schema="core_device")
    # ### end Alembic commands ###



from vortex.handler.TupleDataObservableProxyHandler import (
    TupleDataObservableProxyHandler,
)

from peek_core_device._private.PluginNames import deviceFilt
from peek_core_device._private.PluginNames import deviceObservableName
from peek_plugin_base.PeekVortexUtil import peekServerName


def makeDeviceTupleDataObservableProxy():
    proxy = TupleDataObservableProxyHandler(
        observableName=deviceObservableName,
        proxyToVortexName=peekServerName,
        additionalFilt=deviceFilt,
    )

    return proxy



"""Replaced isOnline with deviceStatus

Peek Plugin Database Migration Script

Revision ID: 7f628c121c4c
Revises: 02f9305f9e7f
Create Date: 2021-03-29 13:29:30.096962

"""

# revision identifiers, used by Alembic.
revision = "7f628c121c4c"
down_revision = "02f9305f9e7f"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column(
        "DeviceInfo",
        sa.Column(
            "deviceStatus",
            sa.SmallInteger(),
            server_default="0",
            nullable=False,
        ),
        schema="core_device",
    )
    op.drop_column("DeviceInfo", "isOnline", schema="core_device")


def downgrade():
    op.add_column(
        "DeviceInfo",
        sa.Column("isOnline", sa.Boolean(), server_default="0", nullable=False),
        schema="core_device",
    )
    op.execute(
        """ 
        UPDATE "core_device"."DeviceInfo" 
        SET "isOnline" = true 
        WHERE "deviceStatus" <> 0 
        """
    )
    op.drop_column("DeviceInfo", "deviceStatus", schema="core_device")



import logging
from datetime import datetime
from typing import List
from typing import Optional

from reactivex import Observable
from reactivex.subject import Subject
from sqlalchemy.orm.exc import NoResultFound
from twisted.internet.defer import Deferred
from vortex.DeferUtil import deferToThreadWrapWithLogger
from vortex.DeferUtil import noMainThread

from peek_core_device._private.server.controller.MainController import (
    MainController,
)
from peek_core_device._private.storage.DeviceInfoTable import DeviceInfoTable
from peek_core_device._private.storage.GpsLocationTable import GpsLocationTable
from peek_core_device.server.DeviceApiABC import DeviceApiABC
from peek_core_device.tuples.DeviceDetailTuple import DeviceDetailTuple
from peek_core_device.tuples.DeviceGpsLocationTuple import (
    DeviceGpsLocationTuple,
)
from peek_core_device.tuples.DeviceStatusTuple import DeviceStatusTuple

logger = logging.getLogger(__name__)


class DeviceApi(DeviceApiABC):
    def __init__(self, mainController: MainController, ormSessionCreator):
        self._mainController = mainController
        self._ormSessionCreator = ormSessionCreator

        self._deviceOnlineSubject = Subject()
        self._deviceGpsLocationSubject = Subject()

    def shutdown(self):
        pass

    @deferToThreadWrapWithLogger(logger)
    def deviceDetails(self, deviceTokens: List[str]) -> Deferred:
        ormSession = self._ormSessionCreator()
        try:
            all = (
                ormSession.query(DeviceInfoTable)
                .filter(DeviceInfoTable.deviceToken.in_(deviceTokens))
                .all()
            )

            tuples = [
                DeviceDetailTuple(
                    deviceToken=d.deviceToken,
                    deviceType=d.deviceType,
                    description=d.description,
                    lastOnline=d.lastOnline,
                    deviceStatus=d.deviceStatus,
                )
                for d in all
            ]

            return tuples

        finally:
            ormSession.close()

    @deferToThreadWrapWithLogger(logger)
    def deviceDescription(self, deviceToken: str) -> Deferred:
        return self.deviceDescriptionBlocking(deviceToken)

    def deviceDescriptionBlocking(self, deviceToken: str) -> Optional[str]:
        noMainThread()

        ormSession = self._ormSessionCreator()
        try:
            all = (
                ormSession.query(DeviceInfoTable)
                .filter(DeviceInfoTable.deviceToken == deviceToken)
                .all()
            )

            if not all:
                return None

            return all[0].description

        finally:
            ormSession.close()

    def deviceOnlineStatus(self) -> Observable:
        return self._deviceOnlineSubject

    def notifyOfOnlineStatus(
        self, deviceId: str, deviceToken: str, status: bool
    ):
        self._deviceOnlineSubject.on_next(
            DeviceStatusTuple(
                deviceToken=deviceToken, deviceId=deviceId, deviceStatus=status
            )
        )

    def deviceCurrentGpsLocation(self) -> Observable:
        return self._deviceGpsLocationSubject

    def notifyCurrentGpsLocation(
        self,
        deviceToken: str,
        latitude: float,
        longitude: float,
        datetime: datetime,
    ):
        self._deviceGpsLocationSubject.on_next(
            DeviceGpsLocationTuple(
                deviceToken=deviceToken,
                latitude=latitude,
                longitude=longitude,
                datetime=datetime,
            )
        )

    @deferToThreadWrapWithLogger(logger)
    def deviceCurrentGpsLocations(
        self, deviceTokens: List[str]
    ) -> List[DeviceGpsLocationTuple]:
        session = self._ormSessionCreator()
        try:
            query = session.query(GpsLocationTable).filter(
                GpsLocationTable.deviceToken.in_(deviceTokens)
            )
            return [d.toTuple() for d in query.all()]
        except NoResultFound:
            return []
        finally:
            session.close()

    @deferToThreadWrapWithLogger(logger)
    def deviceTokens(self):
        ormSession = self._ormSessionCreator()
        query = ormSession.query(DeviceInfoTable)
        return [device.deviceToken for device in query.all()]



from vortex.Tuple import TupleField
from vortex.Tuple import addTupleType
from vortex.TupleAction import TupleActionABC

from peek_core_device._private.PluginNames import deviceTuplePrefix


@addTupleType
class DeviceBackgroundStateTupleAction(TupleActionABC):
    __tupleType__ = deviceTuplePrefix + "DeviceBackgroundStateTupleAction"

    deviceId: str = TupleField()
    deviceBackgrounded: bool = TupleField()



from typing import Optional

from vortex.Tuple import TupleField
from vortex.Tuple import addTupleType
from vortex.TupleAction import TupleActionABC

from peek_core_device._private.PluginNames import deviceTuplePrefix


# I'm using the word Alter here, because UpdateUpdate is confusing.
@addTupleType
class AlterDeviceUpdateAction(TupleActionABC):
    """Alter Device Update Tuple

    This action tuple applies changes to the Update from the admin UI.

    """

    __tupleType__ = deviceTuplePrefix + "AlterDeviceUpdateAction"

    #:  the ID of the DeviceUpdateTuple
    updateId: int = TupleField()

    #: Set the enabled property of the update
    isEnabled: Optional[bool] = TupleField()

    #: Delete the update from the database
    remove: bool = TupleField(False)



from datetime import datetime

from vortex.Tuple import Tuple
from vortex.Tuple import TupleField
from vortex.Tuple import addTupleType

from peek_core_device._private.PluginNames import deviceTuplePrefix
from peek_core_device.tuples.DeviceInfoTuple import DeviceInfoTuple


@addTupleType
class DeviceDetailTuple(Tuple):
    __tupleType__ = deviceTuplePrefix + "DeviceDetailTuple"

    DEVICE_OFFLINE = DeviceInfoTuple.DEVICE_OFFLINE
    DEVICE_ONLINE = DeviceInfoTuple.DEVICE_ONLINE
    DEVICE_BACKGROUND = DeviceInfoTuple.DEVICE_BACKGROUND

    deviceToken: str = TupleField()
    deviceType: str = TupleField()
    description: str = TupleField()
    lastOnline: str = TupleField()
    deviceStatus: int = TupleField()

    def _formatDatetime(self, dt: datetime):
        return dt.strftime("%H:%M, %a, %d %b %Y")

    @property
    def deviceStatusDisplayText(self):
        if self.deviceStatus == DeviceInfoTuple.DEVICE_ONLINE:
            return "Online, App Visible"
        if self.deviceStatus == DeviceInfoTuple.DEVICE_BACKGROUND:
            return "Online, App Backgrounded"
        if self.lastOnline:
            return self._formatDatetime(self.lastOnline)
        return "Never Connected"

    @property
    def lastOnlineDisplayText(self):
        return self._formatDatetime(self.lastOnline)



import datetime as datetime

from vortex.Tuple import Tuple
from vortex.Tuple import TupleField
from vortex.Tuple import addTupleType

from peek_core_device._private.PluginNames import deviceTuplePrefix


@addTupleType
class DeviceGpsLocationTuple(Tuple):
    __tupleType__ = deviceTuplePrefix + "GpsLocationTuple"

    latitude: float = TupleField()
    longitude: float = TupleField()
    datetime: datetime = TupleField()
    deviceId: str = TupleField()
    deviceToken: str = TupleField()



from vortex.Tuple import TupleField
from vortex.Tuple import addTupleType
from vortex.TupleAction import TupleActionABC

from peek_core_device._private.PluginNames import deviceTuplePrefix


@addTupleType
class BandwidthTestTuple(TupleActionABC):
    __tupleType__ = deviceTuplePrefix + "BandwidthTestTuple"

    testData: str = TupleField()



""" 
 * view.common.uiobj.Style.py
 *
 *  Copyright Synerty Pty Ltd 2013
 *
 *  This software is proprietary, you are not free to copy
 *  or redistribute this code in any format.
 *
 *  All rights to this software are reserved by 
 *  Synerty Pty Ltd
 *
"""
import json
import logging

from twisted.web.server import NOT_DONE_YET
from txhttputil.site.BasicResource import BasicResource
from vortex.Payload import Payload

from peek_core_device._private.server.controller.UpdateController import (
    UpdateController,
)
from peek_core_device._private.tuples.CreateDeviceUpdateAction import (
    CreateDeviceUpdateAction,
)

logger = logging.getLogger(name=__name__)


class DeviceUpdateUploadResource(BasicResource):
    isLeaf = True
    useLargeRequest = True

    UPDATE_TYPE_PLATFORM = 0
    UPDATE_TYPE_PLUGIN = 1

    def __init__(self, controller: UpdateController):
        BasicResource.__init__(self)
        self._controller = controller

    def render_GET(self, request):
        raise Exception("Updates must be done via the UI")

    def render_POST(self, request):
        request.responseHeaders.setRawHeaders("content-type", ["text/plain"])
        logger.info("Received device update upload request")

        action: CreateDeviceUpdateAction = (
            Payload().fromVortexMsg(request.args[b"payload"][0]).tuples[0]
        )

        def good(data):
            request.write(json.dumps({"message": str(data)}).encode())
            request.finish()
            logger.info("Finished creating device update %s" % data)

        def bad(failure):
            e = failure.value
            logger.exception(e)

            request.write(
                json.dumps(
                    {
                        "error": str(failure.value),
                        "stdout": e.stdout if hasattr(e, "stdout") else None,
                        "stderr": e.stderr if hasattr(e, "stderr") else None,
                    }
                ).encode()
            )

            request.finish()

        d = self._controller.processCreateUpdateUpload(
            request.content.namedTemporaryFile, action
        )
        d.addCallbacks(good, bad)

        def closedError(failure):
            logger.error("Got closedError %s" % failure)

        request.notifyFinish().addErrback(closedError)

        return NOT_DONE_YET



import logging
from datetime import datetime
from typing import List
from typing import Optional

import pytz
from twisted.internet.defer import Deferred
from twisted.internet.defer import inlineCallbacks
from twisted.internet.task import LoopingCall
from vortex.DeferUtil import vortexLogFailure
from vortex.Tuple import Tuple
from vortex.TupleAction import TupleActionABC

from peek_core_device._private.server.controller.NotifierController import (
    NotifierController,
)
from peek_core_device._private.tuples.BandwidthTestResultTuple import (
    BandwidthTestResultTuple,
)
from peek_plugin_base.LoopingCallUtil import peekCatchErrbackWithLogger
from peek_plugin_base.storage.RunPyInPg import runPyInPg

logger = logging.getLogger(__name__)


class BandwidthResultController:
    INSERT_SECONDS = 120.0

    def __init__(
        self, dbSessionCreator, notifierController: NotifierController
    ):
        self._dbSessionCreator = dbSessionCreator
        self._notifierController = notifierController

        self._metricUpdateQueue = {}
        self._insertLoopingCall = None

    def start(self):
        self._insertLoopingCall = LoopingCall(self._poll)
        d = self._insertLoopingCall.start(self.INSERT_SECONDS)
        d.addErrback(vortexLogFailure, logger)

    def shutdown(self):
        if self._insertLoopingCall and self._insertLoopingCall.running:
            self._insertLoopingCall.stop()

        self._insertLoopingCall = None
        self._notifierController = None
        self._metricUpdateQueue = {}

    def processTupleAction(self, tupleAction: TupleActionABC) -> List[Tuple]:
        if isinstance(tupleAction, BandwidthTestResultTuple):
            self._metricUpdateQueue[tupleAction.deviceToken] = (
                tupleAction.metric
            )
            return []

    @peekCatchErrbackWithLogger(logger)
    @inlineCallbacks
    def _poll(self) -> Optional[Deferred]:
        if not self._metricUpdateQueue:
            return

        toProcess, self._metricUpdateQueue = self._metricUpdateQueue, {}

        startTime = datetime.now(pytz.UTC)
        errors = yield runPyInPg(
            logger, self._dbSessionCreator, self._updateMetric, None, toProcess
        )
        logger.debug(
            "Inserted %s bandwidth metrics in %s",
            len(toProcess),
            datetime.now(pytz.UTC) - startTime,
        )
        for error in errors:
            logger.error(error)

        self._notifierController.notifyAllDeviceInfos()

    @classmethod
    def _updateMetric(cls, plpy, updates: dict[str, int]) -> list[str]:
        """Process Offline Cache Update

        :rtype: Deferred
        """
        errors = []

        plan = plpy.prepare(
            """UPDATE core_device."DeviceInfo"
            SET "lastBandwidthMetric" = $2
            WHERE "deviceToken" = $1;""",
            ["text", "integer"],
        )
        for deviceToken, metric in updates.items():
            try:
                plpy.execute(plan, [deviceToken, metric])
            except plpy.SPIError as e:
                errors.append(
                    f"Failed to make update for {deviceToken}" f": {str(e)}"
                )

        return errors



from vortex.handler.TupleDataObservableHandler import TupleDataObservableHandler

from .SettingPropertyHandler import makeSettingPropertyHandler


def makeAdminBackendHandlers(
    tupleObservable: TupleDataObservableHandler, dbSessionCreator
):
    yield makeSettingPropertyHandler(tupleObservable, dbSessionCreator)
